@@ -1,6 +1,6 @@
 /*
     BCU SDK bcu development enviroment
-    Copyright (C) 2005 Martin Kgler <mkoegler@auto.tuwien.ac.at>
+    Copyright (C) 2005-2011 Martin Koegler <mkoegler@auto.tuwien.ac.at>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #ifndef _BCU_FUNCS_H
@@ -844,11 +844,11 @@ _U_TS_Del (uchar val)
 }
 
 static void inline
-_U_MS_Post (uchar msgid, uchar pointer)
+_U_MS_Post (uchar msgid, uchar_loptr pointer)
 {
-  if (!__builtin_constant_p (pointer))
+  if (!__builtin_constant_p (msgid))
     {
-      if (!__builtin_constant_p (msgid))
+      if (!__builtin_constant_p (pointer))
 	asm volatile ("lda %0\n\tldx %1\n\tjsr U_MS_Post"::"r" (msgid),
 		      "r" (pointer):"A", "X", "RegB", "RegL", "RegM", "RegN");
       else
@@ -857,7 +857,7 @@ _U_MS_Post (uchar msgid, uchar pointer)
     }
   else
     {
-      if (!__builtin_constant_p (msgid))
+      if (!__builtin_constant_p (pointer))
 	asm volatile ("lda $%0\n\tldx %1\n\tjsr U_MS_Post"::"i" (msgid),
 		      "r" (pointer):"A", "X", "RegB", "RegL", "RegM", "RegN");
       else
