@@ -1,6 +1,6 @@
 /*
     BCU SDK bcu development enviroment
-    Copyright (C) 2005 Martin Kgler <mkoegler@auto.tuwien.ac.at>
+    Copyright (C) 2005-2011 Martin Koegler <mkoegler@auto.tuwien.ac.at>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #include <stdlib.h>
@@ -22,6 +22,23 @@
 #include "regexp.h"
 #include "symboltab.h"
 
+bool
+UsePEI (const Device & d)
+{
+  return d.on_pei_init () || d.on_pei_message ()
+    || d.on_pei_cycle ()
+    || d.on_pei_user ()
+    || d.on_pei_rc_even ()
+    || d.on_pei_rc_odd ()
+    || d.on_pei_tc ()
+    || d.on_pei_tdre ()
+    || d.on_pei_sci_idle ()
+    || d.on_pei_spif ()
+    || d.on_pei_oca ()
+    || d.on_pei_ocb () || d.on_pei_ica () || d.on_pei_icb ();
+}
+
+
 int
 UsedbyInterface (const Device & d, const String & name)
 {
@@ -86,7 +103,7 @@ CheckStringParameter (Device & d, StringParameter & o)
   if (o.RegExp ())
     if (!checkRegExp (o.RegExp ()))
       die (_("line %d: invalid regular expression"), o.RegExp_lineno);
-#ifdef PHASE1
+#ifdef CHECK1
   if (!o.Title ())
     undefined ("StringParameter", "Title", o.lineno);
   if (!o.Default ())
@@ -106,7 +123,8 @@ CheckStringParameter (Device & d, StringParameter & o)
     }
   else
     o.ID_lineno = o.lineno;
-#else
+#endif
+#ifdef CHECK2
   if (!o.Value ())
     die (_("line %d: No value defined"), o.lineno);
   if (strlen (o.Value ()) >= o.MaxLength)
@@ -130,7 +148,7 @@ CheckFloatParameter (Device & d, FloatParameter & o)
   if (o.MinValue > o.MaxValue)
     die (_("line %d: MinValue > MaxValue"), o.MaxValue_lineno);
 
-#ifdef PHASE1
+#ifdef CHECK1
   if (!o.Title ())
     undefined ("FloatParameter", "Title", o.lineno);
   if (!o.Default_lineno)
@@ -149,7 +167,8 @@ CheckFloatParameter (Device & d, FloatParameter & o)
     }
   else
     o.ID_lineno = o.lineno;
-#else
+#endif
+#ifdef CHECK2
   if (!o.Value_lineno)
     die (_("line %d: No value defined"), o.lineno);
   if (o.Value < o.MinValue)
@@ -172,7 +191,7 @@ CheckIntParameter (Device & d, IntParameter & o)
   if (o.MinValue > o.MaxValue)
     die (_("line %d: MinValue > MaxValue"), o.MaxValue_lineno);
 
-#ifdef PHASE1
+#ifdef CHECK1
   if (!o.Title ())
     undefined ("IntParameter", "Title", o.lineno);
   if (!o.Default_lineno)
@@ -191,7 +210,8 @@ CheckIntParameter (Device & d, IntParameter & o)
     }
   else
     o.ID_lineno = o.lineno;
-#else
+#endif
+#ifdef CHECK2
   if (!o.Value_lineno)
     die (_("line %d: No value defined"), o.lineno);
   if (o.Value < o.MinValue)
@@ -235,7 +255,7 @@ CheckIntParameter (Device & d, IntParameter & o)
 void
 CheckListParameter (Device & d, ListParameter & o)
 {
-  int i, df = -1;
+  int i, df = -1, vl = -1;
   if (!o.Name ())
     undefined ("ListParameter", "Name", o.lineno);
 
@@ -244,7 +264,7 @@ CheckListParameter (Device & d, ListParameter & o)
   if (!o.Elements ())
     die (_("line %d: no enumeration values"), o.Elements_lineno);
 
-#ifdef PHASE1
+#ifdef CHECK1
   if (!o.Title ())
     undefined ("ListParameter", "Title", o.lineno);
   if (!o.Default ())
@@ -258,6 +278,8 @@ CheckListParameter (Device & d, ListParameter & o)
       o.ListElements[i].Value = o.Elements[i].Value;
       if (o.Elements[i].Name == o.Default)
 	df = i;
+      if (o.Value_lineno && o.Elements[i].Name == o.Value)
+	vl = i;
       o.Elements[i].Value = o.ListElements[i].Name;
     }
   if (df == -1)
@@ -265,6 +287,12 @@ CheckListParameter (Device & d, ListParameter & o)
 
   o.ListDefault = o.Elements[df].Value;
 
+  if (o.Value_lineno && vl == -1)
+    die (_("line %d: unknown value"), o.Value_lineno);
+
+  if (o.Value_lineno)
+    o.Value = o.Elements[vl].Value;
+
   NewSymbol (o.Name + "_t", o.lineno);
   o.ID = NewSymbol (o.Name, o.lineno);
   if (!UsedbyInterface (d, o.Name))
@@ -277,7 +305,8 @@ CheckListParameter (Device & d, ListParameter & o)
     o.ID_lineno = o.lineno;
 
 
-#else
+#endif
+#ifdef CHECK2
   if (!o.Value ())
     undefined ("ListParameter", "Value", o.lineno);
   for (i = 0; i < o.Elements (); i++)
@@ -301,7 +330,7 @@ CheckPollingMaster (Device & d, PollingMaster & o)
   if (!o.Name ())
     undefined ("PollingMaster", "Name", o.lineno);
 
-#ifdef PHASE1
+#ifdef CHECK1
   if (!o.Title ())
     undefined ("PollingMaster", "Title", o.lineno);
 
@@ -312,7 +341,8 @@ CheckPollingMaster (Device & d, PollingMaster & o)
     }
   else
     o.ID_lineno = o.lineno;
-#else
+#endif
+#ifdef CHECK2
   if (!o.PollingAddress_lineno && o.PollingCount_lineno)
     die (_("line %d: missing PollingAddress"), o.lineno);
   if (o.PollingAddress_lineno && !o.PollingCount_lineno)
@@ -336,7 +366,7 @@ CheckPollingSlave (Device & d, PollingSlave & o)
   if (!o.Name ())
     undefined ("PollingSlave", "Name", o.lineno);
 
-#ifdef PHASE1
+#ifdef CHECK1
   if (!o.Title ())
     undefined ("PollingSlave", "Title", o.lineno);
 
@@ -347,7 +377,8 @@ CheckPollingSlave (Device & d, PollingSlave & o)
     }
   else
     o.ID_lineno = o.lineno;
-#else
+#endif
+#ifdef CHECK2
   if (!o.PollingAddress_lineno && o.PollingSlot_lineno)
     die (_("line %d: missing PollingAddress"), o.lineno);
   if (o.PollingAddress_lineno && !o.PollingSlot_lineno)
@@ -405,7 +436,7 @@ CheckProperty (Device & d, Property & o, Object & o1)
       o.eeprom = 0;
     }
 
-#ifdef PHASE1
+#ifdef CHECK1
   if (!o.Title ())
     undefined ("Property", "Title", o.lineno);
 
@@ -413,12 +444,15 @@ CheckProperty (Device & d, Property & o, Object & o1)
   if (!UsedbyInterface (d, o.Name))
     {
       o.ID = 0;
+      o.Disable = 1;
+      o.Disable_lineno = o.lineno;
+      o.ReadOnly = !o.Writeable;
+      o.ReadOnly_lineno = o.lineno;
     }
   else
     o.ID_lineno = o.lineno;
-  o.Disable = 0;
-  o.ReadOnly = !o.Writeable;
-#else
+#endif
+#ifdef CHECK2
 
   if (!o.WriteAccess_lineno)
     {
@@ -436,6 +470,8 @@ CheckProperty (Device & d, Property & o, Object & o1)
   if (o.WriteAccess < 0 || o.WriteAccess > 3)
     die (_("line %d: invalid access level"), o.WriteAccess_lineno);
 
+#endif
+
   if (!o.Disable_lineno)
     {
       o.Disable_lineno = o.lineno;
@@ -447,8 +483,6 @@ CheckProperty (Device & d, Property & o, Object & o1)
       o.ReadOnly_lineno = o.lineno;
       o.ReadOnly = !o.Writeable;
     }
-
-#endif
 }
 
 void
@@ -461,13 +495,14 @@ CheckObject (Device & d, Object & o, int &no)
   if (!o.Name ())
     undefined ("Object", "Name", o.lineno);
 
-#ifdef PHASE1
+#ifdef CHECK1
   o.ObjectIndex = no;
   o.ObjectIndex_lineno = o.lineno;
   no++;
 
   NewSymbol (o.Name, o.lineno);
-#else
+#endif
+#ifdef CHECK2
 
 #endif
 
@@ -485,11 +520,12 @@ CheckObject (Device & d, Object & o, int &no)
       if (o.Propertys[i].PropertyID == o.Propertys[j].PropertyID)
 	die (_("line %d: duplicate property ID with %s"),
 	     o.Propertys[i].lineno, o.Propertys[j].Name ());
-#ifdef PHASE1
+#ifdef CHECK1
   o.PropCount = o.Propertys ();
   o.RAccess = 3;
   o.WAccess = 3;
-#else
+#endif
+#ifdef CHECK2
   ra = 4;
   wa = 4;
   PropCount = 0;
@@ -546,7 +582,7 @@ CheckGroupObject (Device & d, GroupObject & o)
   if (!o.Type_lineno)
     undefined ("GroupObject", "Type", o.lineno);
 
-#ifdef PHASE1
+#ifdef CHECK1
   if (!o.Title ())
     undefined ("GroupObject", "Title", o.lineno);
 
@@ -571,7 +607,8 @@ CheckGroupObject (Device & d, GroupObject & o)
       o.Receiving_lineno = o.lineno;
     }
 
-#else
+#endif
+#ifdef CHECK2
 
   if (!o.Priority_lineno)
     {
@@ -600,7 +637,7 @@ CheckGroupObject (Device & d, GroupObject & o)
       o.eeprom_lineno = 0;
       o.eeprom = 0;
     }
-#ifdef PHASE1
+#ifdef CHECK1
   if (o.Sending)
     NewSymbol (o.Name + "_transmit", o.lineno);
 
@@ -609,7 +646,8 @@ CheckGroupObject (Device & d, GroupObject & o)
 
   if (o.Reading && o.Sending)
     NewSymbol (o.Name + "_clear", o.lineno);
-#else
+#endif
+#ifdef CHECK2
   if (!o.Sending && o.SendAddress_lineno)
     die (_("line %d: can not send on this group object"), o.lineno);
 
@@ -627,7 +665,7 @@ void
 CheckInterface (Device & d, Interface & o)
 {
 
-#ifdef PHASE1
+#ifdef CHECK1
 
   if (!o.Reference_lineno)
     undefined ("Interface", "Reference", o.lineno);
@@ -660,7 +698,8 @@ CheckInterface (Device & d, Interface & o)
   if (o.InvisibleIf_lineno)
     CheckExpressionBool (o.InvisibleIf, o.InvisibleIf_lineno, d);
 
-#else
+#endif
+#ifdef CHECK2
 
 #endif
 
@@ -671,7 +710,7 @@ CheckFunctionalBlock (Device & d, FunctionalBlock & o)
 {
   int i;
 
-#ifdef PHASE1
+#ifdef CHECK1
   if (!o.Title ())
     undefined ("FunctionalBlock", "Title", o.lineno);
 
@@ -684,7 +723,8 @@ CheckFunctionalBlock (Device & d, FunctionalBlock & o)
   if (o.ProfileID <= 0)
     die (_("line %d: invalid ProfileID"), o.ProfileID_lineno);
 
-#else
+#endif
+#ifdef CHECK2
 
 #endif
 
@@ -715,7 +755,7 @@ CheckExpressionList (Expr * s, int l, const Device & d)
       s->id[i] = o.Elements[j].Value;
     }
   s->s = o.ID;
-  if (!o.ID ())
+  if (!o.ID_lineno)
     {
       int found = 0;
       for (i = 0; i < s->id (); i++)
@@ -748,9 +788,9 @@ GetExpressionType (Expr * s, int l, const Device & d)
       if (d.StringParameters () != i)
 	{
 	  s->s = d.StringParameters[i].ID;
-	  if (!s->s ())
+	  if (!d.StringParameters[i].ID_lineno)
 	    {
-	      s->s = d.StringParameters[i].Value;
+	      s->s = d.StringParameters[i].Default;
 	      s->Type = Expr::E_STRING;
 	    }
 	  return EX_STRING;
@@ -761,9 +801,9 @@ GetExpressionType (Expr * s, int l, const Device & d)
       if (d.IntParameters () != i)
 	{
 	  s->s = d.IntParameters[i].ID;
-	  if (!s->s ())
+	  if (!d.IntParameters[i].ID_lineno)
 	    {
-	      s->i = d.IntParameters[i].Value;
+	      s->i = d.IntParameters[i].Default;
 	      s->Type = Expr::E_INT;
 	    }
 	  return EX_INT;
@@ -774,9 +814,9 @@ GetExpressionType (Expr * s, int l, const Device & d)
       if (d.FloatParameters () != i)
 	{
 	  s->s = d.FloatParameters[i].ID;
-	  if (!s->s ())
+	  if (!d.FloatParameters[i].ID_lineno)
 	    {
-	      s->f = d.FloatParameters[i].Value;
+	      s->f = d.FloatParameters[i].Default;
 	      s->Type = Expr::E_FLOAT;
 	    }
 	  return EX_FLOAT;
@@ -1153,11 +1193,12 @@ CheckDevice (Device & d)
   if (!d.A_Event_lineno)
     d.A_Event = 1;
 
-#ifdef PHASE1
+#ifdef CHECK1
   if (!d.Title ())
     undefined ("Device", "Title", d.lineno);
 
-#else
+#endif
+#ifdef CHECK2
   if (!d.PhysicalAddress_lineno)
     undefined ("Device", "PhyiscalAddress", d.lineno);
 #endif
@@ -1173,7 +1214,10 @@ CheckDevice (Device & d)
   if (d.BCU == BCU_bcu12 && d.InstallKey_lineno)
     die (_("installkey not supported"));
   if (d.BCU == BCU_bcu12 && d.Keys ())
-    die (_("bcu1 supports no access protection"));
+    die (_("BCU1 supports no access protection"));
+
+  if (d.BCU == BCU_bcu12 && UsePEI (d))
+    die (_("BCU1 supports no custom PEI handler"));
 
   if (d.BCU != BCU_bcu12)
     {
