@@ -1,6 +1,6 @@
 /*
     BCU SDK bcu development enviroment
-    Copyright (C) 2005 Martin Kgler <mkoegler@auto.tuwien.ac.at>
+    Copyright (C) 2005-2011 Martin Koegler <mkoegler@auto.tuwien.ac.at>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,13 +14,14 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #include "gencode.h"
 #include "map.h"
 #include "addrtab.h"
 #include "loadctl.h"
+#include "check.h"
 
 static const char *
 inttype (int i)
@@ -124,7 +125,7 @@ GenGroupObjectHeader (FILE * f, GroupObject & o)
 
       if (o.Sending)
 	{
-#ifdef PHASE1
+#ifdef BUILDMAX
 	  if (o.Sending)
 #else
 	  if (o.SendAddress_lineno)
@@ -138,7 +139,7 @@ GenGroupObjectHeader (FILE * f, GroupObject & o)
 
       if (o.Reading)
 	{
-#ifdef PHASE1
+#ifdef BUILDMAX
 	  if (o.Reading)
 #else
 	  if (o.ReadRequestAddress_lineno)
@@ -151,7 +152,7 @@ GenGroupObjectHeader (FILE * f, GroupObject & o)
 	}
       if (o.Reading && o.Sending)
 	{
-#ifdef PHASE1
+#ifdef BUILDMAX
 	  if (o.Reading && o.Sending)
 #else
 	  if (o.ReadRequestAddress_lineno && o.SendAddress_lineno)
@@ -246,6 +247,71 @@ GenCommonHeader (FILE * f, Device & d)
     fprintf (f, "static void %s();\n", d.on_run ());
   if (d.on_save ())
     fprintf (f, "static void %s();\n", d.on_save ());
+
+  if (d.on_pei_init ())
+    fprintf (f, "static void %s();\n", d.on_pei_init ());
+  if (d.on_pei_message ())
+    fprintf (f, "static void %s();\n", d.on_pei_message ());
+  if (d.on_pei_cycle ())
+    fprintf (f, "static void %s();\n", d.on_pei_cycle ());
+  if (d.on_pei_user ())
+    fprintf (f, "static void %s(uchar event);\n", d.on_pei_user ());
+  if (d.on_pei_rc_even ())
+    fprintf (f, "static void %s();\n", d.on_pei_rc_even ());
+  if (d.on_pei_rc_odd ())
+    fprintf (f, "static void %s();\n", d.on_pei_rc_odd ());
+  if (d.on_pei_tc ())
+    fprintf (f, "static void %s();\n", d.on_pei_tc ());
+  if (d.on_pei_tdre ())
+    fprintf (f, "static void %s();\n", d.on_pei_tdre ());
+  if (d.on_pei_sci_idle ())
+    fprintf (f, "static void %s();\n", d.on_pei_sci_idle ());
+  if (d.on_pei_spif ())
+    fprintf (f, "static void %s();\n", d.on_pei_spif ());
+  if (d.on_pei_oca ())
+    fprintf (f, "static void %s();\n", d.on_pei_oca ());
+  if (d.on_pei_ocb ())
+    fprintf (f, "static void %s();\n", d.on_pei_ocb ());
+  if (d.on_pei_ica ())
+    fprintf (f, "static void %s();\n", d.on_pei_ica ());
+  if (d.on_pei_icb ())
+    fprintf (f, "static void %s();\n", d.on_pei_icb ());
+
+  if (UsePEI (d))
+    {
+      fprintf (f,
+	       "NOSAVE(void _PEI_Handler_stub())\n{\nuchar event=__tmp_space[0];\n");
+      if (d.on_pei_init ())
+	fprintf (f, "if (event==PM_INIT) %s();\n", d.on_pei_init ());
+      if (d.on_pei_message ())
+	fprintf (f, "if (event==PM_MESSAGE) %s();\n", d.on_pei_message ());
+      if (d.on_pei_cycle ())
+	fprintf (f, "if (event==PM_CYCLE) %s();\n", d.on_pei_cycle ());
+      if (d.on_pei_rc_even ())
+	fprintf (f, "if (event==PM_rc_even) %s();\n", d.on_pei_rc_even ());
+      if (d.on_pei_rc_odd ())
+	fprintf (f, "if (event==PM_rc_odd) %s();\n", d.on_pei_rc_odd ());
+      if (d.on_pei_tc ())
+	fprintf (f, "if (event==PM_rc_tc) %s();\n", d.on_pei_tc ());
+      if (d.on_pei_tdre ())
+	fprintf (f, "if (event==PM_tdre) %s();\n", d.on_pei_tdre ());
+      if (d.on_pei_sci_idle ())
+	fprintf (f, "if (event==PM_sci_idle) %s();\n", d.on_pei_sci_idle ());
+      if (d.on_pei_spif ())
+	fprintf (f, "if (event==PM_spif) %s();\n", d.on_pei_spif ());
+      if (d.on_pei_oca ())
+	fprintf (f, "if (event==PM_OCA) %s();\n", d.on_pei_oca ());
+      if (d.on_pei_ocb ())
+	fprintf (f, "if (event==PM_OCB) %s();\n", d.on_pei_ocb ());
+      if (d.on_pei_ica ())
+	fprintf (f, "if (event==PM_ICA) %s();\n", d.on_pei_ica ());
+      if (d.on_pei_icb ())
+	fprintf (f, "if (event==PM_ICB) %s();\n", d.on_pei_icb ());
+      if (d.on_pei_user ())
+	fprintf (f, "%s(uchar event);\n", d.on_pei_user ());
+      fprintf (f, "}\n\n");
+    }
+
   if (d.on_init ())
     fprintf (f, "NOSAVE(void %s_stub()) {%s();}\n",
 	     d.on_init (), d.on_init ());
@@ -471,7 +537,7 @@ printParameterInfo (FILE * f, Device & d)
   for (i = 0; i < d.StringParameters (); i++)
     {
       StringParameter & s = d.StringParameters[i];
-      if (!s.ID ())
+      if (!s.ID_lineno)
 	continue;
       fprintf (f, "\t.hword %d\n", strlen (s.ID ()) + 7);
       fprintf (f, "\t.hword %d\n", L_STRING_PAR);
@@ -482,7 +548,7 @@ printParameterInfo (FILE * f, Device & d)
   for (i = 0; i < d.FloatParameters (); i++)
     {
       FloatParameter & s = d.FloatParameters[i];
-      if (!s.ID ())
+      if (!s.ID_lineno)
 	continue;
       fprintf (f, "\t.hword %d\n", strlen (s.ID ()) + 5);
       fprintf (f, "\t.hword %d\n", L_FLOAT_PAR);
@@ -492,7 +558,7 @@ printParameterInfo (FILE * f, Device & d)
   for (i = 0; i < d.IntParameters (); i++)
     {
       IntParameter & s = d.IntParameters[i];
-      if (!s.ID ())
+      if (!s.ID_lineno)
 	continue;
       fprintf (f, "\t.hword %d\n", strlen (s.ID ()) + 6);
       fprintf (f, "\t.hword %d\n", L_INT_PAR);
@@ -504,7 +570,7 @@ printParameterInfo (FILE * f, Device & d)
     {
       int j, l = 0;
       ListParameter & s = d.ListParameters[i];
-      if (!s.ID ())
+      if (!s.ID_lineno)
 	continue;
       for (j = 0; j < s.Elements (); j++)
 	l += strlen (s.Elements[j].Value ()) + 1;
@@ -519,7 +585,7 @@ printParameterInfo (FILE * f, Device & d)
   for (i = 0; i < d.GroupObjects (); i++)
     {
       GroupObject & s = d.GroupObjects[i];
-      if (!s.ID ())
+      if (!s.ID_lineno)
 	continue;
       fprintf (f, "\t.hword %d\n", strlen (s.ID ()) + 4);
       fprintf (f, "\t.hword %d\n", L_GROUP_OBJECT);
@@ -596,7 +662,10 @@ GenBCUHeader (FILE * f, Device & d)
       fprintf (f, "\t.hword commobjs\n");
       fprintf (f, "\t.hword ram_start\n");
       fprintf (f, "\t.hword eeprom_start\n");
-      fprintf (f, "\t.hword 0x0000 #SPHandler\n");
+      if (UsePEI (d))
+	fprintf (f, "\t.hword peihandler #SPHandler\n");
+      else
+	fprintf (f, "\t.hword 0x0000 #SPHandler\n");
 
       fprintf (f, "\t.hword _UserInit\n");
       fprintf (f, "\t.hword _UserRun\n");
@@ -723,6 +792,13 @@ GenAsmEntry (FILE * f, Device & d)
   for (i = 0; i < j; i++)
     fprintf (f, "\t.byte 0\n");
 
+  fprintf (f, "\t.section .eibpeihandler\n");
+  if (UsePEI (d))
+    {
+      fprintf (f, "peihandler:\n");
+      fprintf (f, "\tsta __tmp_space\n");
+      fprintf (f, "\tjmp _PEI_Handler_stub\n");
+    }
   fprintf (f, "\t.section .eibobjects\n");
   fprintf (f, "eibobjects:\n");
   for (i = 0; i < d.Objects (); i++)
