@@ -1,6 +1,6 @@
 /*
     EIBD eib bus access and management daemon
-    Copyright (C) 2005 Martin Kgler <mkoegler@auto.tuwien.ac.at>
+    Copyright (C) 2005-2011 Martin Koegler <mkoegler@auto.tuwien.ac.at>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,14 +14,15 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #ifndef TYPES_H
 #define TYPES_H
 
 #include <stdint.h>
-#include "strings.h"
+#include "config.h"
+#include "my_strings.h"
 #include "array.h"
 
 /** unsigned char */
@@ -36,4 +37,8 @@ typedef uint16_t eibaddr_t;
 /** EIB key */
 typedef uint32_t eibkey_type;
 
+#ifdef USE_NOLIBSTDC
+#include "libstdc.h"
+#endif
+
 #endif
