@@ -1,6 +1,6 @@
 /*
     EIBD eib bus access and management daemon
-    Copyright (C) 2005 Martin Kgler <mkoegler@auto.tuwien.ac.at>
+    Copyright (C) 2005-2011 Martin Koegler <mkoegler@auto.tuwien.ac.at>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #ifndef C_PEI16s_H
@@ -27,6 +27,7 @@
 
 #define PEI16s_PREFIX "bcu1s"
 #define PEI16s_CREATE PEI16s_ll_Create
+#define PEI16s_CLEANUP NULL
 
 inline LowLevelDriverInterface *
 PEI16s_ll_Create (const char *dev, Trace * t)
