@@ -1,6 +1,6 @@
 /*
     EIBD eib bus access and management daemon
-    Copyright (C) 2005 Martin Kgler <mkoegler@auto.tuwien.ac.at>
+    Copyright (C) 2005-2011 Martin Koegler <mkoegler@auto.tuwien.ac.at>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,17 +14,17 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #include <string.h>
 #include <netdb.h>
 #include <sys/socket.h>
 #include <unistd.h>
-#include <asm/types.h>
 #include "eibnetip.h"
 #include "config.h"
 #ifdef HAVE_LINUX_NETLINK
+#include <asm/types.h>
 #include <linux/netlink.h>
 #include <linux/rtnetlink.h>
 #endif
@@ -34,6 +34,10 @@
 #include <iphlpapi.h>
 #undef Array
 #endif
+#if HAVE_BSD_SOURCEINFO
+#include <net/if.h>
+#include <net/route.h>
+#endif
 
 int
 GetHostIP (struct sockaddr_in *sock, const char *Name)
@@ -45,6 +49,9 @@ GetHostIP (struct sockaddr_in *sock, const char *Name)
   h = gethostbyname (Name);
   if (!h)
     return 0;
+#ifdef HAVE_SOCKADDR_IN_LEN
+  sock->sin_len = sizeof (*sock);
+#endif
   sock->sin_family = h->h_addrtype;
   sock->sin_addr.s_addr = (*((unsigned long *) h->h_addr_list[0]));
   return 1;
@@ -71,7 +78,8 @@ GetSourceAddress (const struct sockaddr_in *dest, struct sockaddr_in *src)
   if (s == -1)
     return 0;
   req.n.nlmsg_len =
-    NLMSG_SPACE (sizeof (req.r)) + RTA_LENGTH (sizeof (*dest));
+    NLMSG_SPACE (sizeof (req.r)) +
+    RTA_LENGTH (sizeof (dest->sin_addr.s_addr));
   req.n.nlmsg_flags = NLM_F_REQUEST;
   req.n.nlmsg_type = RTM_GETROUTE;
   req.r.rtm_family = AF_INET;
@@ -144,13 +152,91 @@ GetSourceAddress (const struct sockaddr_in *dest, struct sockaddr_in *src)
 }
 #endif
 
+#if HAVE_BSD_SOURCEINFO
+typedef struct
+{
+  struct rt_msghdr hdr;
+  char data[1000];
+} r_req;
+
+#ifndef HAVE_SA_SIZE
+static int
+SA_SIZE (struct sockaddr *sa)
+{
+  int align = sizeof (long);
+  int len = (sa->sa_len ? sa->sa_len : align);
+  if (len & (align - 1))
+    {
+      len += align - (len & (align - 1));
+    }
+  return len;
+}
+#endif
+
+int
+GetSourceAddress (const struct sockaddr_in *dest, struct sockaddr_in *src)
+{
+  int s;
+  r_req req;
+  char *cp = req.data;
+  memset (&req, 0, sizeof (req));
+  memset (src, 0, sizeof (*src));
+  s = socket (PF_ROUTE, SOCK_RAW, 0);
+  if (s == -1)
+    return 0;
+  req.hdr.rtm_msglen = sizeof (req) + sizeof (*dest);
+  req.hdr.rtm_version = RTM_VERSION;
+  req.hdr.rtm_flags = RTF_UP;
+  req.hdr.rtm_type = RTM_GET;
+  req.hdr.rtm_addrs = RTA_DST | RTA_IFP;
+  memcpy (cp, dest, sizeof (*dest));
+  if (write (s, (char *) &req, req.hdr.rtm_msglen) < 0)
+    return 0;
+  if (read (s, (char *) &req, sizeof (req)) < 0)
+    return 0;
+  close (s);
+  int i;
+  cp = (char *) (&req.hdr + 1);
+  for (i = 1; i; i <<= 1)
+    if (i & req.hdr.rtm_addrs)
+      {
+	struct sockaddr *sa = (struct sockaddr *) cp;
+	if (i == RTA_IFA)
+	  {
+	    src->sin_len = sizeof (*src);
+	    src->sin_family = AF_INET;
+	    src->sin_addr.s_addr =
+	      ((struct sockaddr_in *) sa)->sin_addr.s_addr;
+	    return 1;
+	  }
+	cp += SA_SIZE (sa);
+      }
+  return 0;
+}
+#endif
+
+bool
+compareIPAddress (const struct sockaddr_in & a, const struct sockaddr_in & b)
+{
+  if (a.sin_family != AF_INET)
+    return false;
+  if (b.sin_family != AF_INET)
+    return false;
+  if (a.sin_port != b.sin_port)
+    return false;
+  if (a.sin_addr.s_addr != b.sin_addr.s_addr)
+    return false;
+  return true;
+}
+
 EIBNetIPPacket::EIBNetIPPacket ()
 {
   service = 0;
+  memset (&src, 0, sizeof (src));
 }
 
 EIBNetIPPacket *
-EIBNetIPPacket::fromPacket (const CArray & c)
+EIBNetIPPacket::fromPacket (const CArray & c, const struct sockaddr_in src)
 {
   EIBNetIPPacket *p;
   unsigned len;
@@ -164,6 +250,7 @@ EIBNetIPPacket::fromPacket (const CArray & c)
   p = new EIBNetIPPacket;
   p->service = (c[2] << 8) | c[3];
   p->data.set (c.array () + 6, len - 6);
+  p->src = src;
   return p;
 }
 
@@ -184,23 +271,36 @@ EIBNetIPPacket::ToPacket ()
 }
 
 CArray
-IPtoEIBNetIP (const struct sockaddr_in * a)
+IPtoEIBNetIP (const struct sockaddr_in * a, bool nat)
 {
   CArray buf;
   buf.resize (8);
   buf[0] = 0x08;
   buf[1] = 0x01;
-  buf[2] = (ntohl (a->sin_addr.s_addr) >> 24) & 0xff;
-  buf[3] = (ntohl (a->sin_addr.s_addr) >> 16) & 0xff;
-  buf[4] = (ntohl (a->sin_addr.s_addr) >> 8) & 0xff;
-  buf[5] = (ntohl (a->sin_addr.s_addr) >> 0) & 0xff;
-  buf[6] = (ntohs (a->sin_port) >> 8) & 0xff;
-  buf[7] = (ntohs (a->sin_port) >> 0) & 0xff;
+  if (nat)
+    {
+      buf[2] = 0;
+      buf[3] = 0;
+      buf[4] = 0;
+      buf[5] = 0;
+      buf[6] = 0;
+      buf[7] = 0;
+    }
+  else
+    {
+      buf[2] = (ntohl (a->sin_addr.s_addr) >> 24) & 0xff;
+      buf[3] = (ntohl (a->sin_addr.s_addr) >> 16) & 0xff;
+      buf[4] = (ntohl (a->sin_addr.s_addr) >> 8) & 0xff;
+      buf[5] = (ntohl (a->sin_addr.s_addr) >> 0) & 0xff;
+      buf[6] = (ntohs (a->sin_port) >> 8) & 0xff;
+      buf[7] = (ntohs (a->sin_port) >> 0) & 0xff;
+    }
   return buf;
 }
 
 int
-EIBnettoIP (const CArray & buf, struct sockaddr_in *a)
+EIBnettoIP (const CArray & buf, struct sockaddr_in *a,
+	    const struct sockaddr_in *src, bool & nat)
 {
   int ip, port;
   memset (a, 0, sizeof (*a));
@@ -208,9 +308,22 @@ EIBnettoIP (const CArray & buf, struct sockaddr_in *a)
     return 1;
   ip = (buf[2] << 24) | (buf[3] << 16) | (buf[4] << 8) | (buf[5]);
   port = (buf[6] << 8) | (buf[7]);
+#ifdef HAVE_SOCKADDR_IN_LEN
+  a->sin_len = sizeof (*a);
+#endif
   a->sin_family = AF_INET;
-  a->sin_port = htons (port);
-  a->sin_addr.s_addr = htonl (ip);
+  if (port == 0)
+    a->sin_port = src->sin_port;
+  else
+    a->sin_port = htons (port);
+  if (ip == 0)
+    {
+      nat = true;
+      a->sin_addr.s_addr = src->sin_addr.s_addr;
+    }
+  else
+    a->sin_addr.s_addr = htonl (ip);
+
   return 0;
 }
 
@@ -219,7 +332,7 @@ EIBNetIPSocket::EIBNetIPSocket (struct sockaddr_in bindaddr, bool reuseaddr,
 {
   int i;
   t = tr;
-  t->TracePrintf (0, this, "Open");
+  TRACEPRINTF (t, 0, this, "Open");
   multicast = 0;
   pth_sem_init (&insignal);
   pth_sem_init (&outsignal);
@@ -227,48 +340,65 @@ EIBNetIPSocket::EIBNetIPSocket (struct sockaddr_in bindaddr, bool reuseaddr,
   memset (&maddr, 0, sizeof (maddr));
   memset (&sendaddr, 0, sizeof (sendaddr));
   memset (&recvaddr, 0, sizeof (recvaddr));
+  memset (&recvaddr2, 0, sizeof (recvaddr2));
   recvall = 0;
 
   fd = socket (AF_INET, SOCK_DGRAM, 0);
   if (fd == -1)
-    throw Exception (DEV_OPEN_FAIL);
+    return;
 
   if (reuseaddr)
     {
       i = 1;
       if (setsockopt (fd, SOL_SOCKET, SO_REUSEADDR, &i, sizeof (i)) == -1)
-	throw Exception (DEV_OPEN_FAIL);
+	{
+	  close (fd);
+	  fd = -1;
+	  return;
+	}
     }
   if (bind (fd, (struct sockaddr *) &bindaddr, sizeof (bindaddr)) == -1)
-    throw Exception (DEV_OPEN_FAIL);
+    {
+      close (fd);
+      fd = -1;
+      return;
+    }
 
   Start ();
-  t->TracePrintf (0, this, "Openend");
+  TRACEPRINTF (t, 0, this, "Openend");
 }
 
 EIBNetIPSocket::~EIBNetIPSocket ()
 {
-  t->TracePrintf (0, this, "Close");
+  TRACEPRINTF (t, 0, this, "Close");
   Stop ();
   pth_event_free (getwait, PTH_FREE_THIS);
   if (fd != -1)
     {
       if (multicast)
-	setsockopt (fd, SOL_IP, IP_DROP_MEMBERSHIP, &maddr, sizeof (maddr));
+	setsockopt (fd, IPPROTO_IP, IP_DROP_MEMBERSHIP, &maddr,
+		    sizeof (maddr));
       close (fd);
     }
 }
 
-void
+bool
+EIBNetIPSocket::init ()
+{
+  return fd != -1;
+}
+
+bool
 EIBNetIPSocket::SetMulticast (struct ip_mreq multicastaddr)
 {
   if (multicast)
-    throw Exception (DEV_OPEN_FAIL);
+    return false;
   maddr = multicastaddr;
-  if (setsockopt (fd, SOL_IP, IP_ADD_MEMBERSHIP, &maddr, sizeof (maddr)) ==
-      -1)
-    throw Exception (DEV_OPEN_FAIL);
+  if (setsockopt (fd, IPPROTO_IP, IP_ADD_MEMBERSHIP, &maddr, sizeof (maddr))
+      == -1)
+    return false;
   multicast = 1;
+  return true;
 }
 
 void
@@ -307,6 +437,7 @@ void
 EIBNetIPSocket::Run (pth_sem_t * stop1)
 {
   int i;
+  int error = 0;
   uchar buf[255];
   socklen_t rl;
   sockaddr_in r;
@@ -323,11 +454,12 @@ EIBNetIPSocket::Run (pth_sem_t * stop1)
       if (i > 0 && rl == sizeof (r))
 	{
 	  if (recvall == 1 || !memcmp (&r, &recvaddr, sizeof (r)) ||
-	      (recvall == 2 && memcmp (&r, &localaddr, sizeof (r))))
+	      (recvall == 2 && memcmp (&r, &localaddr, sizeof (r))) ||
+	      (recvall == 3 && !memcmp (&r, &recvaddr2, sizeof (r))))
 	    {
 	      t->TracePacket (0, this, "Recv", i, buf);
 	      EIBNetIPPacket *p =
-		EIBNetIPPacket::fromPacket (CArray (buf, i));
+		EIBNetIPPacket::fromPacket (CArray (buf, i), r);
 	      if (p)
 		{
 		  outqueue.put (*p);
@@ -350,6 +482,16 @@ EIBNetIPSocket::Run (pth_sem_t * stop1)
 	    {
 	      pth_sem_dec (&insignal);
 	      inqueue.get ();
+	      error = 0;
+	    }
+	  else
+	    error++;
+	  if (error > 5)
+	    {
+	      t->TracePacket (0, this, "Drop EIBnetSocket", p);
+	      pth_sem_dec (&insignal);
+	      inqueue.get ();
+	      error = 0;
 	    }
 	}
     }
@@ -361,6 +503,7 @@ EIBnet_ConnectRequest::EIBnet_ConnectRequest ()
 {
   memset (&caddr, 0, sizeof (caddr));
   memset (&daddr, 0, sizeof (daddr));
+  nat = false;
 }
 
 EIBNetIPPacket EIBnet_ConnectRequest::ToPacket ()CONST
@@ -370,8 +513,8 @@ EIBNetIPPacket EIBnet_ConnectRequest::ToPacket ()CONST
   CArray
     ca,
     da;
-  ca = IPtoEIBNetIP (&caddr);
-  da = IPtoEIBNetIP (&daddr);
+  ca = IPtoEIBNetIP (&caddr, nat);
+  da = IPtoEIBNetIP (&daddr, nat);
   p.service = CONNECTION_REQUEST;
   p.data.resize (ca () + da () + 1 + CRI ());
   p.data.setpart (ca, 0);
@@ -389,9 +532,9 @@ parseEIBnet_ConnectRequest (const EIBNetIPPacket & p,
     return 1;
   if (p.data () < 18)
     return 1;
-  if (EIBnettoIP (CArray (p.data.array (), 8), &r.caddr))
+  if (EIBnettoIP (CArray (p.data.array (), 8), &r.caddr, &p.src, r.nat))
     return 1;
-  if (EIBnettoIP (CArray (p.data.array () + 8, 8), &r.daddr))
+  if (EIBnettoIP (CArray (p.data.array () + 8, 8), &r.daddr, &p.src, r.nat))
     return 1;
   if (p.data () - 16 != p.data[16])
     return 1;
@@ -402,6 +545,7 @@ parseEIBnet_ConnectRequest (const EIBNetIPPacket & p,
 EIBnet_ConnectResponse::EIBnet_ConnectResponse ()
 {
   memset (&daddr, 0, sizeof (daddr));
+  nat = false;
   channel = 0;
   status = 0;
 }
@@ -411,14 +555,20 @@ EIBNetIPPacket EIBnet_ConnectResponse::ToPacket ()CONST
   EIBNetIPPacket
     p;
   CArray
-    da = IPtoEIBNetIP (&daddr);
+    da = IPtoEIBNetIP (&daddr, nat);
   p.service = CONNECTION_RESPONSE;
-  p.data.resize (da () + CRD () + 3);
+  if (status != 0)
+    p.data.resize (2);
+  else
+    p.data.resize (da () + CRD () + 3);
   p.data[0] = channel;
   p.data[1] = status;
-  p.data.setpart (da, 2);
-  p.data[da () + 2] = CRD () + 1;
-  p.data.setpart (CRD, da () + 3);
+  if (status == 0)
+    {
+      p.data.setpart (da, 2);
+      p.data[da () + 2] = CRD () + 1;
+      p.data.setpart (CRD, da () + 3);
+    }
   return p;
 }
 
@@ -428,9 +578,19 @@ parseEIBnet_ConnectResponse (const EIBNetIPPacket & p,
 {
   if (p.service != CONNECTION_RESPONSE)
     return 1;
+  if (p.data () < 2)
+    return 1;
+  if (p.data[1] != 0)
+    {
+      if (p.data () != 2)
+	return 1;
+      r.channel = p.data[0];
+      r.status = p.data[1];
+      return 0;
+    }
   if (p.data () < 12)
     return 1;
-  if (EIBnettoIP (CArray (p.data.array () + 2, 8), &r.daddr))
+  if (EIBnettoIP (CArray (p.data.array () + 2, 8), &r.daddr, &p.src, r.nat))
     return 1;
   if (p.data () - 10 != p.data[10])
     return 1;
@@ -443,6 +603,7 @@ parseEIBnet_ConnectResponse (const EIBNetIPPacket & p,
 EIBnet_ConnectionStateRequest::EIBnet_ConnectionStateRequest ()
 {
   memset (&caddr, 0, sizeof (caddr));
+  nat = false;
   channel = 0;
 }
 
@@ -451,7 +612,7 @@ EIBNetIPPacket EIBnet_ConnectionStateRequest::ToPacket ()CONST
   EIBNetIPPacket
     p;
   CArray
-    ca = IPtoEIBNetIP (&caddr);
+    ca = IPtoEIBNetIP (&caddr, nat);
   p.service = CONNECTIONSTATE_REQUEST;
   p.data.resize (ca () + 2);
   p.data[0] = channel;
@@ -468,7 +629,7 @@ parseEIBnet_ConnectionStateRequest (const EIBNetIPPacket & p,
     return 1;
   if (p.data () != 10)
     return 1;
-  if (EIBnettoIP (CArray (p.data.array () + 2, 8), &r.caddr))
+  if (EIBnettoIP (CArray (p.data.array () + 2, 8), &r.caddr, &p.src, r.nat))
     return 1;
   r.channel = p.data[0];
   return 0;
@@ -507,6 +668,7 @@ parseEIBnet_ConnectionStateResponse (const EIBNetIPPacket & p,
 EIBnet_DisconnectRequest::EIBnet_DisconnectRequest ()
 {
   memset (&caddr, 0, sizeof (caddr));
+  nat = false;
   channel = 0;
 }
 
@@ -515,7 +677,7 @@ EIBNetIPPacket EIBnet_DisconnectRequest::ToPacket ()CONST
   EIBNetIPPacket
     p;
   CArray
-    ca = IPtoEIBNetIP (&caddr);
+    ca = IPtoEIBNetIP (&caddr, nat);
   p.service = DISCONNECT_REQUEST;
   p.data.resize (ca () + 2);
   p.data[0] = channel;
@@ -532,7 +694,7 @@ parseEIBnet_DisconnectRequest (const EIBNetIPPacket & p,
     return 1;
   if (p.data () != 10)
     return 1;
-  if (EIBnettoIP (CArray (p.data.array () + 2, 8), &r.caddr))
+  if (EIBnettoIP (CArray (p.data.array () + 2, 8), &r.caddr, &p.src, r.nat))
     return 1;
   r.channel = p.data[0];
   return 0;
@@ -638,9 +800,80 @@ parseEIBnet_TunnelACK (const EIBNetIPPacket & p, EIBnet_TunnelACK & r)
   return 0;
 }
 
+EIBnet_ConfigRequest::EIBnet_ConfigRequest ()
+{
+  channel = 0;
+  seqno = 0;
+}
+
+EIBNetIPPacket EIBnet_ConfigRequest::ToPacket ()CONST
+{
+  EIBNetIPPacket
+    p;
+  p.service = DEVICE_CONFIGURATION_REQUEST;
+  p.data.resize (CEMI () + 4);
+  p.data[0] = 4;
+  p.data[1] = channel;
+  p.data[2] = seqno;
+  p.data[3] = 0;
+  p.data.setpart (CEMI, 4);
+  return p;
+}
+
+int
+parseEIBnet_ConfigRequest (const EIBNetIPPacket & p, EIBnet_ConfigRequest & r)
+{
+  if (p.service != DEVICE_CONFIGURATION_REQUEST)
+    return 1;
+  if (p.data () < 6)
+    return 1;
+  if (p.data[0] != 4)
+    return 1;
+  r.channel = p.data[1];
+  r.seqno = p.data[2];
+  r.CEMI.set (p.data.array () + 4, p.data () - 4);
+  return 0;
+}
+
+EIBnet_ConfigACK::EIBnet_ConfigACK ()
+{
+  channel = 0;
+  seqno = 0;
+  status = 0;
+}
+
+EIBNetIPPacket EIBnet_ConfigACK::ToPacket ()CONST
+{
+  EIBNetIPPacket
+    p;
+  p.service = DEVICE_CONFIGURATION_ACK;
+  p.data.resize (4);
+  p.data[0] = 4;
+  p.data[1] = channel;
+  p.data[2] = seqno;
+  p.data[3] = status;
+  return p;
+}
+
+int
+parseEIBnet_ConfigACK (const EIBNetIPPacket & p, EIBnet_ConfigACK & r)
+{
+  if (p.service != DEVICE_CONFIGURATION_ACK)
+    return 1;
+  if (p.data () != 4)
+    return 1;
+  if (p.data[0] != 4)
+    return 1;
+  r.channel = p.data[1];
+  r.seqno = p.data[2];
+  r.status = p.data[3];
+  return 0;
+}
+
 EIBnet_DescriptionRequest::EIBnet_DescriptionRequest ()
 {
   memset (&caddr, 0, sizeof (caddr));
+  nat = false;
 }
 
 EIBNetIPPacket EIBnet_DescriptionRequest::ToPacket ()CONST
@@ -648,7 +881,7 @@ EIBNetIPPacket EIBnet_DescriptionRequest::ToPacket ()CONST
   EIBNetIPPacket
     p;
   CArray
-    ca = IPtoEIBNetIP (&caddr);
+    ca = IPtoEIBNetIP (&caddr, nat);
   p.service = DESCRIPTION_REQUEST;
   p.data = ca;
   return p;
@@ -662,7 +895,7 @@ parseEIBnet_DescriptionRequest (const EIBNetIPPacket & p,
     return 1;
   if (p.data () != 8)
     return 1;
-  if (EIBnettoIP (p.data, &r.caddr))
+  if (EIBnettoIP (p.data, &r.caddr, &p.src, r.nat))
     return 1;
   return 0;
 }
@@ -751,6 +984,7 @@ parseEIBnet_DescriptionResponse (const EIBNetIPPacket & p,
 EIBnet_SearchRequest::EIBnet_SearchRequest ()
 {
   memset (&caddr, 0, sizeof (caddr));
+  nat = false;
 }
 
 EIBNetIPPacket EIBnet_SearchRequest::ToPacket ()CONST
@@ -758,7 +992,7 @@ EIBNetIPPacket EIBnet_SearchRequest::ToPacket ()CONST
   EIBNetIPPacket
     p;
   CArray
-    ca = IPtoEIBNetIP (&caddr);
+    ca = IPtoEIBNetIP (&caddr, nat);
   p.service = SEARCH_REQUEST;
   p.data = ca;
   return p;
@@ -771,7 +1005,7 @@ parseEIBnet_SearchRequest (const EIBNetIPPacket & p, EIBnet_SearchRequest & r)
     return 1;
   if (p.data () != 8)
     return 1;
-  if (EIBnettoIP (p.data, &r.caddr))
+  if (EIBnettoIP (p.data, &r.caddr, &p.src, r.nat))
     return 1;
   return 0;
 }
@@ -794,7 +1028,7 @@ EIBNetIPPacket EIBnet_SearchResponse::ToPacket ()CONST
   EIBNetIPPacket
     p;
   CArray
-    ca = IPtoEIBNetIP (&caddr);
+    ca = IPtoEIBNetIP (&caddr, nat);
   p.service = SEARCH_RESPONSE;
   p.data.resize (64 + services () * 2);
   p.data.setpart (ca, 0);
@@ -829,7 +1063,7 @@ parseEIBnet_SearchResponse (const EIBNetIPPacket & p,
     return 1;
   if (p.data () < 64)
     return 1;
-  if (EIBnettoIP (CArray (p.data.array () + 0, 8), &r.caddr))
+  if (EIBnettoIP (CArray (p.data.array () + 0, 8), &r.caddr, &p.src, r.nat))
     return 1;
   if (p.data[8] != 54)
     return 1;
