@@ -1,6 +1,6 @@
 /*
     EIBD eib bus access and management daemon
-    Copyright (C) 2005 Martin Kgler <mkoegler@auto.tuwien.ac.at>
+    Copyright (C) 2005-2011 Martin Koegler <mkoegler@auto.tuwien.ac.at>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #ifndef SERVER_H
@@ -39,9 +39,13 @@ protected:
     /** server socket */
   int fd;
 
+  virtual void setupConnection (int cfd);
+
     Server (Layer3 * l3, Trace * tr);
 public:
     virtual ~ Server ();
+
+  virtual bool init () = 0;
     /** deregister client connection */
   bool deregister (ClientConnection * con);
 };
