@@ -1,6 +1,6 @@
 /*
     EIBD eib bus access and management daemon
-    Copyright (C) 2005 Martin Kgler <mkoegler@auto.tuwien.ac.at>
+    Copyright (C) 2005-2011 Martin Koegler <mkoegler@auto.tuwien.ac.at>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #include <stdio.h>
@@ -23,13 +23,13 @@
 #include "trace.h"
 
 void
-Trace::TracePacket (int layer, void *inst, const char *msg, int Len,
-		    const uchar * data)
+Trace::TracePacketUncond (int layer, void *inst, const char *msg, int Len,
+			  const uchar * data)
 {
   int i;
-  if (!(layers & (1 << layer)))
-    return;
-  printf ("Layer %d(%08X) %s(%03d):", layer, (unsigned int)inst, msg, Len);
+  int t = time (0);
+  printf ("Layer %d(%08X,%08X) %s(%03d):", layer, (unsigned long) inst, t,
+	  msg, Len);
   for (i = 0; i < Len; i++)
     printf (" %02X", data[i]);
   printf ("\n");
@@ -39,11 +39,46 @@ void
 Trace::TracePrintf (int layer, void *inst, const char *msg, ...)
 {
   va_list ap;
-  if (!(layers & (1 << layer)))
-    return;
-  printf ("Layer %d(%08X) ", layer, (unsigned int)inst);
+  int t = time (0);
+  printf ("Layer %d(%08X,%08X) ", layer, (unsigned long) inst, t);
   va_start (ap, msg);
   vprintf (msg, ap);
   printf ("\n");
   va_end (ap);
 }
+
+void
+Trace::ErrorPrintfUncond (unsigned int msgid, const char *msg, ...)
+{
+  va_list ap;
+  int t = time (0);
+  char c;
+  switch ((msgid >> 28) & 0x0f)
+    {
+    case LEVEL_FATAL:
+      c = 'F';
+      break;
+    case LEVEL_CRITICAL:
+      c = 'C';
+      break;
+    case LEVEL_ERROR:
+      c = 'E';
+      break;
+    case LEVEL_WARNING:
+      c = 'W';
+      break;
+    case LEVEL_NOTICE:
+      c = 'N';
+      break;
+    case LEVEL_INFO:
+      c = 'I';
+      break;
+    default:
+      c = '?';
+    }
+  fprintf (stderr, "%c%08d: ", c, (msgid & 0xffffff));
+  va_start (ap, msg);
+  vfprintf (stderr, msg, ap);
+  fprintf (stderr, "\n");
+  va_end (ap);
+}
