@@ -1,6 +1,6 @@
 /*
     EIBD eib bus access and management daemon
-    Copyright (C) 2005 Martin Kgler <mkoegler@auto.tuwien.ac.at>
+    Copyright (C) 2005-2011 Martin Koegler <mkoegler@auto.tuwien.ac.at>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #ifndef C_FT12_H
@@ -27,9 +27,10 @@
 #define FT12_DOC "ft12 connects over a serial line without any driver with the FT1.2 Protocol to a BCU 2\n\n"
 #define FT12_PREFIX "ft12"
 #define FT12_CREATE ft12_Create
+#define FT12_CLEANUP NULL
 
 inline Layer2Interface *
-ft12_Create (const char *dev, Trace * t)
+ft12_Create (const char *dev, int flags, Trace * t)
 {
   return new EMI2Layer2Interface (new FT12LowLevelDriver (dev, t), t);
 }
