@@ -1,6 +1,6 @@
 /*
     EIBD eib bus access and management daemon
-    Copyright (C) 2005 Martin Kgler <mkoegler@auto.tuwien.ac.at>
+    Copyright (C) 2005-2011 Martin Koegler <mkoegler@auto.tuwien.ac.at>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #ifndef C_USB_H
@@ -22,17 +22,19 @@
 
 #include "eibusb.h"
 #include "usbif.h"
+#include "usb.h"
 
 #define USB_URL "usb:[bus[:device[:config[:interface]]]]\n"
 #define USB_DOC "usb connects over a KNX USB interface\n\n"
 #define USB_PREFIX "usb"
 #define USB_CREATE Usb_Create
+#define USB_CLEANUP USBEnd
 
 inline Layer2Interface *
-Usb_Create (const char *dev, Trace * t)
+Usb_Create (const char *dev, int flags, Trace * t)
 {
-  usb_set_debug (0);
-  usb_init (0);
+  if (!USBInit (t))
+    return 0;
   return new USBLayer2Interface (new USBLowLevelDriver (dev, t), t);
 }
 
